
#define EXTIO_EXPORTS		1
#define HWNAME				"Demo-1.00"
#define HWMODEL				"Demo ExtIO"
#define SETTINGS_IDENTIFIER	"Demo-1.x"
#define NUM_CARRIER         2
#define LO_MIN				100000LL
#define LO_MAX				7500000000LL
#define LO_PRECISION		5000L
#define EXT_BLOCKLEN		512			/* only multiples of 512 */
#define BORLAND				0

#include "ExtIO_Demo.h"

//---------------------------------------------------------------------------
// #define WIN32_LEAN_AND_MEAN             // Selten verwendete Teile der Windows-Header nicht einbinden.
#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

//---------------------------------------------------------------------------

#pragma warning(disable : 4996)

#define snprintf	_snprintf

static bool SDR_supports_settings = false;  // assume not supported
static bool SDR_settings_valid = false;		// assume settings are for some other ExtIO

static char SDR_progname[32+1] = "\0";
static int  SDR_ver_major = -1;
static int  SDR_ver_minor = -1;

static unsigned	gCustomSamplerate = 48000;
static int		gHwType = exthwUSBfloat32;


static int		giExtSrateIdx = 0;
static unsigned gExtSampleRate = 48000;
static double	gaCarrierFreq[NUM_CARRIER] = { 5000.0, 10000.0 };	// Hz
static double	gaCarrierLevel[NUM_CARRIER] = { -40.0, -20.0 };		// dB
volatile double	gaCarrierAmp[NUM_CARRIER];
volatile double	gaCarrierPhaseInc[NUM_CARRIER];

volatile int64_t	glLOfreq = 0L;
bool	gbInitHW = false;
int		giAttIdx = 0;
int		giDefaultAttIdx = 4;	// 0 dB
int		giMgcIdx = 0;
int		giDefaultMgcIdx = 0;	// 0 dB
int		giAgcIdx = 0;
int		giDefaultAgcIdx = 1;	// Auto
int		giThrIdx = 0;
int		giDefaultThrIdx = 2;	// Threshold: 20 dB
int		giWhatIdx = 0;

pfnExtIOCallback	pfnCallback = 0;
volatile bool	gbExitThread = false;
volatile bool	gbThreadRunning = false;
volatile int	giParameterUsed = 0;
volatile int	giParameterSetNo = 0;

//---------------------------------------------------------------------------

static void setAmpVal()
{
	float attVal;
	float gainVal;
	GetAttenuators( giAttIdx, &attVal );

	//ExtIoGetMGCs(giMgcIdx, &gainVal);
	// do not use ExtIoGetMGCs() - in case agc or thr is on!
	switch (giMgcIdx)
	{
	case 0:		gainVal = 0.0F;	break;
	case 1:		gainVal = 3.0F;	break;
	case 2:		gainVal = 6.0F;	break;
	}

	for ( int k = 0; k < NUM_CARRIER; ++k )
	{
		if (   gaCarrierLevel[k] + attVal + gainVal < -400.0
			|| gaCarrierFreq[k] <= -0.5 * gExtSampleRate
			|| gaCarrierFreq[k] >=  0.5 * gExtSampleRate
			)
		{
			gaCarrierAmp[k] = 0.0;
			gaCarrierPhaseInc[k] = 0.0;
		}
		else
		{
			gaCarrierAmp[k] = pow( 10.0, ( gaCarrierLevel[k] + attVal + gainVal ) / 20.0 );
			// f = dphi / 2*PI * fs  <=>  dphi = 2*PI*(f / fs)
			gaCarrierPhaseInc[k] = 2.0 * 3.1415926535897932384626433832795 * gaCarrierFreq[k] / (double)gExtSampleRate;
		}
	}
}

//---------------------------------------------------------------------------

DWORD WINAPI GeneratorThreadProc( __in  LPVOID lpParameter )
{
	double	aLocalCarrierAmp[NUM_CARRIER];
	double	aLocalCarrierPhaseInc[NUM_CARRIER];
	double	aLocalCarrierPhase[NUM_CARRIER];
	unsigned LocalSampleRate;

	DWORD lTicks = GetTickCount();
	DWORD lTicksNew = 0;
	DWORD lTicksPPM = lTicks;
	unsigned long generatorCount = 0;

	{
		for ( int k = 0; k < NUM_CARRIER; ++k )
			aLocalCarrierPhase[k] = 0.0;

		LocalSampleRate = gExtSampleRate;
	}

	while ( !gbExitThread )
	{
		SleepEx( 1, FALSE );
		if ( gbExitThread )
			break;

		// new parametrization?
		if ( giParameterUsed != giParameterSetNo )
		{
			++giParameterUsed;
			setAmpVal();
			for ( int k = 0; k < NUM_CARRIER; ++k )
			{
				aLocalCarrierAmp[k] = gaCarrierAmp[k];
				aLocalCarrierPhaseInc[k] = gaCarrierPhaseInc[k];
			}
			LocalSampleRate = gExtSampleRate;
		}

		lTicksNew = GetTickCount();
		unsigned long elapsedMs = (unsigned long)( lTicksNew - lTicks );
		generatorCount += ( LocalSampleRate * elapsedMs + 500L ) / 1000L;

		while ( generatorCount >= EXT_BLOCKLEN && !gbExitThread )
		{
			float samplesFlt[ EXT_BLOCKLEN * 2 ];
			memset( &samplesFlt[0], 0, EXT_BLOCKLEN * 2 * sizeof(float) );

			for ( int k = 0; k < NUM_CARRIER; ++k )
			{
				if ( aLocalCarrierAmp[k] <= 0.0 )
					continue;

				const double ampFactor = aLocalCarrierAmp[k];
				const double phaseInc  = aLocalCarrierPhaseInc[k];
				double phase = aLocalCarrierPhase[k];

				unsigned i, j = 0;
				for ( i = j = 0; i < EXT_BLOCKLEN; ++i )
				{
					samplesFlt[ j++ ] += (float)( ampFactor * cos( phase ) );
					samplesFlt[ j++ ] += (float)( ampFactor * sin( phase ) );

					phase += phaseInc;
					if      ( phase >  3.1415926535897932384626433832795 )	phase -= 2.0 * 3.1415926535897932384626433832795;
					else if ( phase < -3.1415926535897932384626433832795 )	phase += 2.0 * 3.1415926535897932384626433832795;
				}

				aLocalCarrierPhase[k] = phase;
			}

			generatorCount -= EXT_BLOCKLEN;

			if (pfnCallback && !gbExitThread)
			{
				if (gHwType == exthwUSBfloat32)
					pfnCallback(EXT_BLOCKLEN, 0, 0.0F, &samplesFlt[0]);
				else
				{
					int   samplePCM[EXT_BLOCKLEN * 2];
					if (gHwType == exthwFullPCM32)
					{
						for (int k = 0; k < 2 * EXT_BLOCKLEN; ++k)
							samplePCM[k] = (int)(samplesFlt[k] * 2147483648.0F);
					}
					else if (gHwType == exthwUSBdata32)
					{
						for (int k = 0; k < 2 * EXT_BLOCKLEN; ++k)
							samplePCM[k] = (int)( samplesFlt[k] * 8388608.0F );
					}
					else if (gHwType == exthwUSBdata24)
					{
						// error!
						for (int k = 0; k < 2 * EXT_BLOCKLEN; ++k)
							samplePCM[k] = (int)( samplesFlt[k] * 8388608.0F );
					}
					else if (gHwType == exthwUSBdataS8)
					{
						signed char * samplesS8 = (signed char *)((void*)(&samplePCM[0]));
						for (int k = 0; k < 2 * EXT_BLOCKLEN; ++k)
						{
							int pcm = (int)(samplesFlt[k] * 128.0F);
							samplesS8[k] = (pcm < -128) ? -128 : ( (pcm > 127) ? 127 : pcm );
						}
					}
					else if (gHwType == exthwUSBdataU8)
					{
						unsigned char * samplesU8 = (unsigned char *)((void*)(&samplePCM[0]));
						for (int k = 0; k < 2 * EXT_BLOCKLEN; ++k)
						{
							int pcm = (int)(samplesFlt[k] * 128.0F) + 128;
							samplesU8[k] = (pcm < 0) ? 0 : ((pcm > 255) ? 255 : pcm);
						}
					}
					pfnCallback(EXT_BLOCKLEN, 0, 0.0F, &samplePCM[0]);
				}
			}
		}

		lTicks = lTicksNew;
	}
	gbExitThread = false;
	gbThreadRunning = false;
	return 0;
}


static void stopThread()
{
	if ( gbThreadRunning )
	{
		gbExitThread = true;
		while ( gbThreadRunning )
		{
			SleepEx( 10, FALSE );
		}
	}
}

static void startThread()
{
	gbExitThread = false;
	gbThreadRunning = true;
	++giParameterSetNo;

	CreateThread( NULL	// LPSECURITY_ATTRIBUTES lpThreadAttributes
		, (SIZE_T)(64 * 1024)	// SIZE_T dwStackSize
		, GeneratorThreadProc	// LPTHREAD_START_ROUTINE lpStartAddress
		, NULL					// LPVOID lpParameter
		, 0						// DWORD dwCreationFlags
		, NULL					// LPDWORD lpThreadId
		);
}


//---------------------------------------------------------------------------

#if BORLAND
//---------------------------------------------------------------------------
//   Important note about DLL memory management when your DLL uses the
//   static version of the RunTime Library:
//
//   If your DLL exports any functions that pass String objects (or structs/
//   classes containing nested Strings) as parameter or function results,
//   you will need to add the library MEMMGR.LIB to both the DLL project and
//   any other projects that use the DLL.  You will also need to use MEMMGR.LIB
//   if any other projects which use the DLL will be performing new or delete
//   operations on any non-TObject-derived classes which are exported from the
//   DLL. Adding MEMMGR.LIB to your project will change the DLL and its calling
//   EXE's to use the BORLNDMM.DLL as their memory manager.  In these cases,
//   the file BORLNDMM.DLL should be deployed along with your DLL.
//
//   To avoid using BORLNDMM.DLL, pass string information using "char *" or
//   ShortString parameters.
//
//   If your DLL uses the dynamic version of the RTL, you do not need to
//   explicitly add MEMMGR.LIB as this will be done implicitly for you
//---------------------------------------------------------------------------

#pragma argsused
BOOL WINAPI DllMain(HINSTANCE hModule, DWORD ul_reason_for_call, LPVOID lpReserved)

#else
BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
#endif
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
	return TRUE;
}

//---------------------------------------------------------------------------
extern "C"
bool __declspec(dllexport) __stdcall InitHW(char *name, char *model, int& type)
{
	type = gHwType;
	strcpy(name,  HWNAME);
	strcpy(model, HWMODEL);

	if ( !gbInitHW )
	{
		// do initialization

		glLOfreq = 6075000L;	// just a default value
		// .......... init here the hardware controlled by the DLL
		// ......... init here the DLL graphical interface, if any

		giAttIdx = giDefaultAttIdx;
		giMgcIdx = giDefaultMgcIdx;
		giAgcIdx = giDefaultAgcIdx;
		giThrIdx = giDefaultThrIdx;

		setAmpVal();

		gbInitHW = true;
	}
	return gbInitHW;
}

//---------------------------------------------------------------------------
extern "C"
bool EXTIO_API OpenHW(void)
{
	// .... display here the DLL panel ,if any....
	// .....if no graphical interface, delete the following statement
	//::SetWindowPos(F->handle, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);

	if ( pfnCallback )
		pfnCallback( -1, extHw_Changed_ATT, 0.0F, 0 );

	// in the above statement, F->handle is the window handle of the panel displayed 
	// by the DLL, if such a panel exists
	return gbInitHW;
}

//---------------------------------------------------------------------------
extern "C"
int  EXTIO_API StartHW(long LOfreq)
{
	int64_t ret = StartHW64( (int64_t)LOfreq );
	return (int)ret;
}

//---------------------------------------------------------------------------
extern "C"
int64_t EXTIO_API StartHW64(int64_t LOfreq)
{
	if (!gbInitHW)
		return 0;

	stopThread();

	SetHWLO64(LOfreq);

	startThread();

	// number of complex elements returned each
	// invocation of the callback routine
	return EXT_BLOCKLEN;
}

//---------------------------------------------------------------------------
extern "C"
void EXTIO_API StopHW(void)
{
	stopThread();
	return;  // nothing to do with this specific HW
}

//---------------------------------------------------------------------------
extern "C"
void EXTIO_API CloseHW(void)
{
	// ..... here you can shutdown your graphical interface, if any............
	if (gbInitHW )
	{
		/* close port */
	}
	gbInitHW = false;
}

//---------------------------------------------------------------------------
extern "C"
int  EXTIO_API SetHWLO(long LOfreq)
{
	int64_t ret = SetHWLO64( (int64_t)LOfreq );
	return (ret & 0xFFFFFFFF);
}

extern "C"
int64_t EXTIO_API SetHWLO64(int64_t LOfreq)
{
	// ..... set here the LO frequency in the controlled hardware
	// Set here the frequency of the controlled hardware to LOfreq
	const int64_t wishedLO = LOfreq;
	int64_t ret = 0;

	// calculate nearest possible frequency
	// - emulate receiver which don't have 1 Hz resolution
	LOfreq += LO_PRECISION / 2;
	LOfreq /= LO_PRECISION;
	LOfreq *= LO_PRECISION;

	// same LO - but user wanted change?
	if ( LOfreq == glLOfreq )
	{
		if ( wishedLO < glLOfreq )
			LOfreq -= LO_PRECISION;
		else if ( wishedLO > glLOfreq )
			LOfreq += LO_PRECISION;
	}

	// check limits
	if ( LOfreq < LO_MIN )
	{
		LOfreq = LO_MIN;
		ret = -LO_MIN;
	}
	else if ( LOfreq > LO_MAX )
	{
		LOfreq = LO_MAX;
		ret = LO_MAX;
	}

	// take frequency
	glLOfreq = LOfreq;

	if ( gbInitHW )
	{
		// tune to that frequency
		// @TODO: recalc / modify carrier frequencies???
		// int64_t err = wishedLO - glLOfreq;
	}

	if ( wishedLO != LOfreq  &&  pfnCallback )
		pfnCallback( -1, extHw_Changed_LO, 0.0F, 0 );

	// 0 Thefunctiondidcompletewithouterrors.
	// <0(anegativenumberN)
	//     Thespecifiedfrequency is lowerthantheminimumthatthehardware iscapabletogenerate.
	//     TheabsolutevalueofNindicateswhatistheminimumsupportedbytheHW.
	// >0(apositivenumberN)Thespecifiedfrequencyisgreaterthanthemaximumthatthehardware
	//     iscapabletogenerate.
	//     ThevalueofNindicateswhatisthemaximumsupportedbytheHW.
	return ret;
}

//---------------------------------------------------------------------------
extern "C"
int  EXTIO_API GetStatus(void)
{
	return 0;  // status not supported by this specific HW,
}

//---------------------------------------------------------------------------
extern "C"
void EXTIO_API SetCallback( pfnExtIOCallback funcptr )
{
	pfnCallback = funcptr;
	return;
}


//---------------------------------------------------------------------------
extern "C"
long EXTIO_API GetHWLO(void)
{
	return (long)( glLOfreq & 0xFFFFFFFF );
}

extern "C"
int64_t EXTIO_API GetHWLO64(void)
{
	return glLOfreq;
}

//---------------------------------------------------------------------------
extern "C"
long EXTIO_API GetHWSR(void)
{
	// This DLL controls just an oscillator, not a digitizer
	return gExtSampleRate;
}


//---------------------------------------------------------------------------

// extern "C" long EXTIO_API GetTune(void);
// extern "C" void EXTIO_API GetFilters(int& loCut, int& hiCut, int& pitch);
// extern "C" char EXTIO_API GetMode(void);
// extern "C" void EXTIO_API ModeChanged(char mode);
// extern "C" void EXTIO_API IFLimitsChanged(long low, long high);
// extern "C" void EXTIO_API TuneChanged(long freq);

// extern "C" void    EXTIO_API TuneChanged64(int64_t freq);
// extern "C" int64_t EXTIO_API GetTune64(void);
// extern "C" void    EXTIO_API IFLimitsChanged64(int64_t low, int64_t high);

//---------------------------------------------------------------------------

// extern "C" void EXTIO_API RawDataReady(long samprate, int *Ldata, int *Rdata, int numsamples)

//---------------------------------------------------------------------------
extern "C"
void EXTIO_API VersionInfo(const char * progname, int ver_major, int ver_minor)
{
  SDR_progname[0] = 0;
  SDR_ver_major = -1;
  SDR_ver_minor = -1;

  if ( progname )
  {
    strncpy( SDR_progname, progname, sizeof(SDR_progname) -1 );
    SDR_ver_major = ver_major;
    SDR_ver_minor = ver_minor;

	// possibility to check program's capabilities
	// depending on SDR program name and version,
	// f.e. if specific extHWstatusT enums are supported
  }
}

//---------------------------------------------------------------------------

// following "Attenuator"s visible on "RF" button

extern "C"
int EXTIO_API GetAttenuators( int atten_idx, float * attenuation )
{
	// fill in attenuation
	// use positive attenuation levels if signal is amplified (LNA)
	// use negative attenuation levels if signal is attenuated
	// sort by attenuation: use idx 0 for highest attenuation / most damping
	// this functions is called with incrementing idx
	//    - until this functions return != 0 for no more attenuator setting

	switch ( atten_idx )
	{
	case 0:		*attenuation = -30.0F;	return 0;
	case 1:		*attenuation = -20.0F;	return 0;
	case 2:		*attenuation = -10.0F;	return 0;
	case 3:		*attenuation = -6.0F;	return 0;
	case 4:		*attenuation =  0.0F;	return 0;
	case 5:		*attenuation =  9.0F;	return 0;
	default:	return 1;
	}
	return 1;
}

extern "C"
int EXTIO_API GetActualAttIdx(void)
{
	return giAttIdx;	// returns -1 on error
}

extern "C"
int EXTIO_API SetAttenuator( int atten_idx )
{
	int iPrevAttIdx = giAttIdx;

	switch ( atten_idx )
	{
	case 0:
	case 1:
	case 2:
	case 3:
	case 4:
	case 5:
		giAttIdx = atten_idx;
		if ( iPrevAttIdx != giAttIdx )
			++giParameterSetNo;
		return 0;
	default:
		return 1;	// ERROR
	}
	return 1;	// ERROR
}


//---------------------------------------------------------------------------

// optional function to get AGC Mode: AGC_OFF (always agc_index = 0), AGC_SLOW, AGC_MEDIUM, AGC_FAST, ...
// this functions is called with incrementing idx
//    - until this functions returns != 0, which means that all agc modes are already delivered

extern "C"
int EXTIO_API ExtIoGetAGCs(int agc_idx, char * text)	// text limited to max 16 char
{
	switch (agc_idx)
	{
	case 0:		strcpy(text, "MGC");	return 0;
	case 1:		strcpy(text, "AGC");	return 0;
	case 2:		strcpy(text, "Thr");	return 0;
	//case 3:		strcpy(text, "?");		return 0;
	default:	return 1;
	}
	return 1;
}


extern "C"
int EXTIO_API ExtIoGetActualAGCidx(void)
{
	return giAgcIdx;	// returns -1 on error
}

extern "C"
int EXTIO_API ExtIoSetAGC(int agc_idx)
{
	// returns != 0 on error
	int iPrevAgcIdx = giAgcIdx;

	switch (agc_idx)
	{
	case 0:
	case 1:
	case 2:
	//case 3:
		giAgcIdx = agc_idx;
		if (iPrevAgcIdx != giAgcIdx)
		{
			++giParameterSetNo;
			if (pfnCallback )
				EXTIO_STATUS_CHANGE(pfnCallback, extHw_Changed_RF_IF);
		}
		return 0;
	default:
		return 1;	// ERROR
	}
	return 1;	// ERROR
}

// optional: HDSDR >= 2.62
extern "C"
int EXTIO_API ExtIoShowMGC(int agc_idx)		// return 1, to continue showing MGC slider on AGC
// return 0, is default for not showing MGC slider
{
	switch (agc_idx)
	{
	case 0:	return 1;	// MGC
	case 1:	return 1;	// AGC
	case 2:	return 1;	// Thr
	//case 3:	return 1;	// ?
	default:
		return 0;	// ERROR
	}
	return 0;	// ERROR
}

//---------------------------------------------------------------------------

// following "MGC"s visible on "IF" button

extern "C"
int EXTIO_API ExtIoGetMGCs(int mgc_idx, float * gain)
{
	// fill in gain
	// sort by ascending gain: use idx 0 for lowest gain
	// this functions is called with incrementing idx
	//    - until this functions returns != 0, which means that all gains are already delivered

	switch (giAgcIdx)
	{
	case 0:	// MGC
		switch (mgc_idx)
		{
		case 0:		*gain = 0.0F;	return 0;
		case 1:		*gain = 3.0F;	return 0;
		case 2:		*gain = 6.0F;	return 0;
		default:	return 1;
		}
		break;
	case 1:	// AGC
		//return 1;
		switch (mgc_idx)	// set threshold!
		{
		case 0:		*gain = 0.0F;	return 0;
		case 1:		*gain = 10.0F;	return 0;
		case 2:		*gain = 20.0F;	return 0;
		case 3:		*gain = 30.0F;	return 0;
		case 4:		*gain = 40.0F;	return 0;
		default:	return 1;
		}
		break;
	case 2:	// Thr
		switch (mgc_idx)
		{
		case 0:		*gain = 10.0F;	return 0;
		case 1:		*gain = 20.0F;	return 0;
		case 2:		*gain = 30.0F;	return 0;
		case 3:		*gain = 40.0F;	return 0;
		case 4:		*gain = 50.0F;	return 0;
		default:	return 1;
		}
		break;
	case 3:	// ?
		switch (mgc_idx)
		{
		case 0:		*gain = 50.0F;	return 0;
		case 1:		*gain = 60.0F;	return 0;
		default:	return 1;
		}
		break;
	}
	return 1;
}

extern "C"
int EXTIO_API ExtIoGetActualMgcIdx(void)
{
	switch (giAgcIdx)
	{
	case 0:	// MGC
		return giMgcIdx;	// returns -1 on error
	case 1:	// AGC
		//return -1;
		return giThrIdx;
	case 2:	// Thr
		return giThrIdx;
	case 3:
		return giWhatIdx;
	}
	return -1;
}

extern "C"
int EXTIO_API ExtIoSetMGC(int mgc_idx)
{
	int iPrevMgcIdx = giMgcIdx;
	int iPrevThrIdx = giThrIdx;

	switch (giAgcIdx)
	{
	case 0:	// MGC
		switch (mgc_idx)
		{
		case 0:
		case 1:
		case 2:
			giMgcIdx = mgc_idx;
			if (iPrevMgcIdx != giMgcIdx)
				++giParameterSetNo;
			return 0;
		default:
			return 1;	// ERROR
		}
		break;

	case 1:	// AGC
		//break;

	case 2:	// Thr
		switch (mgc_idx)
		{
		case 0:
		case 1:
		case 2:
		case 3:
		case 4:
			giThrIdx = mgc_idx;
			if (iPrevMgcIdx != giThrIdx)
				++giParameterSetNo;
			return 0;
		default:
			return 1;	// ERROR
		}
		break;
	case 3:	// ?
		switch (mgc_idx)
		{
		case 0:
		case 1:
			giWhatIdx = mgc_idx;
			return 0;
		default:
			return 1;	// ERROR
		}
		break;
	}
	return 1;	// ERROR
}

//---------------------------------------------------------------------------

extern "C"
int EXTIO_API ExtIoGetSrates( int srate_idx, double * samplerate )
{
	switch ( srate_idx )
	{
	case 0:		*samplerate =   48000.0;	return 0;
	case 1:		*samplerate =   96000.0;	return 0;
	case 2:		*samplerate =  192000.0;	return 0;
	case 3:		*samplerate =  384000.0;	return 0;
	case 4:		*samplerate =  768000.0;	return 0;
	case 5:		*samplerate = 1536000.0;	return 0;
	case 6:		*samplerate = 2400000.0;	return 0;
	case 7:		*samplerate = 3072000.0;	return 0;
	case 8:		*samplerate = 6144000.0;	return 0;
	case 9:		*samplerate = gCustomSamplerate;	return 0;
	default:	return 1;	// ERROR
	}
	return 1;	// ERROR
}

extern "C"
int  EXTIO_API ExtIoGetActualSrateIdx(void)
{
	return giExtSrateIdx;
}

extern "C"
int  EXTIO_API ExtIoSetSrate( int srate_idx )
{
	double newSrate = 0.0;
	if ( 0 == ExtIoGetSrates( srate_idx, &newSrate ) )
	{
		giExtSrateIdx = srate_idx;
		gExtSampleRate = (unsigned)( newSrate + 0.5 );
		++giParameterSetNo;
		return 0;
	}
	return 1;	// ERROR
}

extern "C"
long EXTIO_API ExtIoGetBandwidth( int srate_idx )
{
	double newSrate = 0.0;
	long ret = -1L;
	if ( 0 == ExtIoGetSrates( srate_idx, &newSrate ) )
	{
		switch ( srate_idx )
		{
			case 0:		ret = 40000L;	break;
			case 1:		ret = 80000L;	break;
			case 2:		ret = 160000L;	break;
			case 3:		ret = 320000L;	break;
			case 4:		ret = 640000L;	break;
			case 5:		ret = 1280000L;	break;
			case 6:		ret = 2000000L;	break;
			case 7:		ret = 2560000L;	break;
			case 8:		ret = 5120000L;	break;
			case 9:		ret = (long)(gCustomSamplerate * 0.8);	break;
			default:	ret = -1L;		break;
		}
		return ( ret >= newSrate || ret <= 0L ) ? -1L : ret;
	}
	return -1L;	// ERROR
}

//---------------------------------------------------------------------------

extern "C"
int  EXTIO_API ExtIoGetSetting( int idx, char * description, char * value )
{
	const char * hwTypeStr = 0;
	switch (gHwType)
	{
	default:
	case exthwUSBfloat32:	hwTypeStr = "FLOAT";	break;
	case exthwUSBdata24:	hwTypeStr = "PCM24";	break;
	case exthwUSBdata32:	hwTypeStr = "PCM2432";	break;
	case exthwFullPCM32:	hwTypeStr = "PCM32";	break;
	case exthwUSBdataU8:	hwTypeStr = "PCMU8";	break;
	case exthwUSBdataS8:	hwTypeStr = "PCMS8";	break;
	}

	switch ( idx )
	{
	case 0: snprintf( description, 1024, "%s", "Identifier" );		snprintf( value, 1024, "%s", SETTINGS_IDENTIFIER );	return 0;
	case 1:	snprintf( description, 1024, "%s", "SampleRateIdx" );	snprintf( value, 1024, "%d", giExtSrateIdx );		return 0;
	case 2:	snprintf( description, 1024, "%s", "AttenuationIdx" );	snprintf( value, 1024, "%d", giAttIdx );			return 0;
	case 3:	snprintf( description, 1024, "%s", "0_Freq_Hz" );		snprintf( value, 1024, "%.3f", gaCarrierFreq[0] );	return 0;
	case 4:	snprintf( description, 1024, "%s", "0_Level_dB" );		snprintf( value, 1024, "%.3f", gaCarrierLevel[0] );	return 0;
	case 5:	snprintf( description, 1024, "%s", "1_Freq_Hz" );		snprintf( value, 1024, "%.3f", gaCarrierFreq[1] );	return 0;
	case 6:	snprintf( description, 1024, "%s", "1_Level_dB" );		snprintf( value, 1024, "%.3f", gaCarrierLevel[1] );	return 0;
	case 7:	snprintf( description, 1024, "%s", "SampleType FLOAT/PCM24/PCM2432/PCM32/PCMU8/PCMS8" );		snprintf( value, 1024, "%s", hwTypeStr );	return 0;
	case 8:	snprintf( description, 1024, "%s", "SampleRate Hz" );	snprintf( value, 1024, "%u", gCustomSamplerate );	return 0;
	default:	return -1;	// ERROR
	}
	return -1;	// ERROR
}


extern "C"
void EXTIO_API ExtIoSetSetting( int idx, const char * value )
{
	double newSrate;
	float  newAtten = 0.0F;
	int tempInt;
	// now we know that there's no need to save our settings into some (.ini) file,
	// what won't be possible without admin rights!!!,
	// if the program (and ExtIO) is installed in C:\Program files\..
	SDR_supports_settings = true;
	if ( idx != 0 && !SDR_settings_valid )
		return;	// ignore settings for some other ExtIO

	switch ( idx )
	{
	case 0:		SDR_settings_valid = ( value && !strcmp( value, SETTINGS_IDENTIFIER ) );
				// make identifier version specific??? - or not ==> never change order of idx!
				break;
	case 1:		tempInt = atoi( value );
				if ( 0 == ExtIoGetSrates( tempInt, &newSrate ) )
				{
					giExtSrateIdx = tempInt;
					gExtSampleRate = (unsigned)( newSrate + 0.5 );
				}
				break;
	case 2:		tempInt = atoi( value );
				if ( 0 == GetAttenuators( tempInt,&newAtten ) )
					giDefaultAttIdx = giAttIdx = tempInt;
				break;
	case 3:		gaCarrierFreq[0]  = atof(value);	break;
	case 4:		gaCarrierLevel[0] = atof(value);	break;
	case 5:		gaCarrierFreq[1]  = atof(value);	break;
	case 6:		gaCarrierLevel[1] = atof(value);	break;
	case 7:		if (!strcmp(value, "FLOAT") || !strcmp(value, "float") || !strcmp(value, "FLT") || !strcmp(value, "flt") || !strcmp(value, "FLOAT32") || !strcmp(value, "float32"))
					gHwType = exthwUSBfloat32;
				else if (!strcmp(value, "PCM24") || !strcmp(value, "pcm24"))
					gHwType = exthwUSBdata24;
				else if (!strcmp(value, "PCM2432") || !strcmp(value, "pcm2432"))
					gHwType = exthwUSBdata32;
				else if (!strcmp(value, "PCM32") || !strcmp(value, "pcm32"))
					gHwType = exthwFullPCM32;
				else if (!strcmp(value, "PCMU8") || !strcmp(value, "pcmu8") || !strcmp(value, "PCM8") || !strcmp(value, "pcm8"))
					gHwType = exthwUSBdataU8;
				else if (!strcmp(value, "PCMS8") || !strcmp(value, "pcms8"))
					gHwType = exthwUSBdataS8;
				else
					gHwType = exthwUSBfloat32;
				break;
	case 8:		if ( atoi(value) > 0 )
					gCustomSamplerate = atoi(value);
				break;
	}

}

//---------------------------------------------------------------------------
